#!/usr/bin/env python
#_*_coding:utf-8_*_

__all__ = ['readFasta','saveCode', 'checkFasta',
		   'AAC', 'EAAC', 'CKSAAP', 'DPC', 'TPC', 'DDE', 'BINARY', 'BLOSUM62', 'AAINDEX', 'ZSCALE',
		   'GAAC', 'EGAAC', 'CKSAAGP', 'GDPC', 'GTPC',
		   'NMBroto', 'Moran', 'Geary',
		   'CTDC', 'CTDT', 'CTDD',
		   'CTriad', 'KSCTriad',
		   'SOCNumber', 'QSOrder',
		   'PAAC', 'APAAC',
		   'KNNprotein', 'KNNpeptide',
		   'PSSM', 'SSEC', 'SSEB', 'Disorder', 'DisorderC', 'DisorderB', 'ASA', 'TA',
		   'NeedlemanWunsch',
		   ]